@@ -62,7 +62,9 @@ import java.util.Set;
 import java.util.SortedSet;
 import java.util.TreeSet;
 import java.util.UUID;
+import javax.servlet.http.HttpServletResponse;
 import com.amazonaws.AmazonClientException;
+import com.amazonaws.AmazonServiceException;
 import com.amazonaws.Request;
 import com.amazonaws.handlers.RequestHandler;
 import com.amazonaws.services.s3.AmazonS3Client;
@@ -86,11 +88,11 @@ import com.amazonaws.services.s3.model.UploadPartRequest;
 import com.amazonaws.services.s3.model.UploadPartResult;
 import com.amazonaws.util.TimingInfo;
 import com.openexchange.exception.OXException;
+import com.openexchange.filestore.FileStorage;
+import com.openexchange.filestore.FileStorageCodes;
 import com.openexchange.java.Streams;
 import com.openexchange.java.Strings;
 import com.openexchange.java.util.UUIDs;
-import com.openexchange.tools.file.external.FileStorage;
-import com.openexchange.tools.file.external.FileStorageCodes;
 
 /**
  * {@link S3FileStorage}
@@ -204,7 +206,7 @@ public class S3FileStorage implements FileStorage {
                 }
             }
         } finally {
-            Streams.close(chunk, chunkedUpload);
+            Streams.close(chunk, chunkedUpload, file);
         }
         return removePrefix(key);
     }
@@ -318,40 +320,45 @@ public class S3FileStorage implements FileStorage {
 
     @Override
     public long appendToFile(InputStream file, String name, long offset) throws OXException {
-        /*
-         * TODO: This would be more efficient using the "CopyPartRequest", which is not yet supported by ceph
-         * http://ceph.com/docs/next/radosgw/s3/#features-support
-         */
-        /*
-         * get existing object
-         */
-        String key = addPrefix(name);
-        S3Object s3Object = getObject(key);
-        long currentLength = s3Object.getObjectMetadata().getContentLength();
-        if (currentLength != offset) {
-            throw FileStorageCodes.INVALID_OFFSET.create(Long.valueOf(offset), name, Long.valueOf(currentLength));
-        }
-        /*
-         * append both streams at temporary location
-         */
-        String tempName = null;
-        SequenceInputStream inputStream = null;
         try {
-            inputStream = new SequenceInputStream(s3Object.getObjectContent(), file);
-            tempName = saveNewFile(inputStream);
+            /*
+                 * TODO: This would be more efficient using the "CopyPartRequest", which is not yet supported by ceph
+                 * http://ceph.com/docs/next/radosgw/s3/#features-support
+                 */
+            /*
+             * get existing object
+             */
+            String tempName = null;
+            String key = addPrefix(name);
+            S3Object s3Object = null;
+            SequenceInputStream inputStream = null;
+            try {
+                s3Object = getObject(key);
+                long currentLength = s3Object.getObjectMetadata().getContentLength();
+                if (currentLength != offset) {
+                    throw FileStorageCodes.INVALID_OFFSET.create(Long.valueOf(offset), name, Long.valueOf(currentLength));
+                }
+                /*
+                 * append both streams at temporary location
+                 */
+                inputStream = new SequenceInputStream(s3Object.getObjectContent(), file);
+                tempName = saveNewFile(inputStream);
+            } finally {
+                Streams.close(inputStream, s3Object);
+            }
+            /*
+             * replace old file, cleanup
+             */
+            try {
+                String tempKey = addPrefix(tempName);
+                amazonS3.copyObject(bucketName, tempKey, bucketName, key);
+                amazonS3.deleteObject(bucketName, tempKey);
+                return getMetadata(key).getContentLength();
+            } catch (AmazonClientException e) {
+                throw wrap(e, key);
+            }
         } finally {
-            Streams.close(inputStream);
-        }
-        /*
-         * replace old file, cleanup
-         */
-        try {
-            String tempKey = addPrefix(tempName);
-            amazonS3.copyObject(bucketName, tempKey, bucketName, key);
-            amazonS3.deleteObject(bucketName, tempKey);
-            return getMetadata(key).getContentLength();
-        } catch (AmazonClientException e) {
-            throw wrap(e, key);
+            Streams.close(file);
         }
     }
 
@@ -393,11 +400,24 @@ public class S3FileStorage implements FileStorage {
 
     @Override
     public InputStream getFile(String name, long offset, long length) throws OXException {
+        long fileSize = getFileSize(name);
+        if (offset > fileSize || -1 != length && length > fileSize - offset) {
+            throw FileStorageCodes.INVALID_RANGE.create(offset, length, name, fileSize);
+        }
         String key = addPrefix(name);
-        GetObjectRequest request = new GetObjectRequest(bucketName, key).withRange(offset, offset + length - 1);
+        GetObjectRequest request = new GetObjectRequest(bucketName, key);
+        if (-1 != length) {
+            request.setRange(offset, offset + length - 1);
+        } else {
+            request.setRange(offset, fileSize - 1);
+        }
         try {
             return amazonS3.getObject(request).getObjectContent();
         } catch (AmazonClientException e) {
+            if (AmazonServiceException.class.isInstance(e) &&
+                HttpServletResponse.SC_REQUESTED_RANGE_NOT_SATISFIABLE == ((AmazonServiceException) e).getStatusCode()) {
+                throw FileStorageCodes.INVALID_RANGE.create(e, offset, length, name, fileSize);
+            }
             throw wrap(e, key);
         }
     }
